AUI.Combat = {}
AUI.Combat.Minimeter = {}
AUI.Combat.Statistics = {}
AUI.Combat.Text = {}
AUI.Combat.DamageMeter = {}

--Combat
AUI_COMBAT_DATA_TYPE_NONE = 1
AUI_COMBAT_DATA_TYPE_DAMAGE_OUT = 2
AUI_COMBAT_DATA_TYPE_HEAL_OUT = 3
AUI_COMBAT_DATA_TYPE_DAMAGE_IN = 4
AUI_COMBAT_DATA_TYPE_HEAL_IN = 5

AUI_COMBAT_DATA_TYPE_DAMAGE = 10
AUI_COMBAT_DATA_TYPE_HEAL = 12

local isLoaded = false
local isCombatMeterStarted = false
local ultiReady = false
local potionReady = false
local playerName = GetRawUnitName(AUI_PLAYER_UNIT_TAG)
local idList = {}
local abilityProcEffects = {}
local playerData = nil
local combatMeterStoppedTime = GetGameTimeMilliseconds()
local isPlayer = 
{
	[COMBAT_UNIT_TYPE_PLAYER] 			= true,
	[COMBAT_UNIT_TYPE_PLAYER_PET] 		= true,
}

local isGroup= 
{
	[COMBAT_UNIT_TYPE_GROUP] 			= true,
}

local isDamage = 
{
	[ACTION_RESULT_DAMAGE] 				= true,
	[ACTION_RESULT_CRITICAL_DAMAGE] 	= true,
	[ACTION_RESULT_DOT_TICK]            = true,
	[ACTION_RESULT_DOT_TICK_CRITICAL]   = true,
	[ACTION_RESULT_FALL_DAMAGE]         = true,
	[ACTION_RESULT_DAMAGE_SHIELDED] 	= true,
	[ACTION_RESULT_BLOCKED_DAMAGE]		= true,
}

local isCrit = 
{
	[ACTION_RESULT_CRITICAL_DAMAGE] 	= true,
	[ACTION_RESULT_DOT_TICK_CRITICAL]   = true,
	[ACTION_RESULT_CRITICAL_HEAL]       = true,
	[ACTION_RESULT_HOT_TICK_CRITICAL]   = true,	
}

 local isHeal = {
	[ACTION_RESULT_HEAL]                = true,
	[ACTION_RESULT_CRITICAL_HEAL]       = true,
	[ACTION_RESULT_HOT_TICK]            = true,
	[ACTION_RESULT_HOT_TICK_CRITICAL]   = true,	
}

local experienceData = {}

AUI.Combat.Data = {}

local function UpdateUI()
	if AUI.Combat.Minimeter.IsEnabled() then
		AUI.Combat.Minimeter.UpdateAll()
	end
	
	if AUI.Combat.DamageMeter.IsShow() then
		AUI.Combat.DamageMeter.Update(AUI.Combat.Data)
	end		
end

local function IsPlayer(_sourceType)
	if isPlayer[_sourceType] then
		return true
	end
	
	return false
end

local function IsGroup(_sourceType)
	if isGroup[_sourceType] then
		return true
	end
	
	return false
end

local function ClearCombatData()
	AUI.Combat.Data = {}

	for _, data in pairs(idList) do	
		if not IsPlayer(data.unitType) and not IsGroup(data.unitType) then
			data= nil
		end
	end
	
	playerData = nil
	
	AUI.ResetMiniMeter()
end

local function IsCrit(_result)
	if isCrit[_result] then
			return true
	end
	
	return false
end

local function UpdateData(_value, _isCrit, _data, _ms)
	if _data and _ms then
		_data.endTimeMS = _ms - _data.startTimeMS

		if _value and _value > 0 then
			_data.total = _data.total + _value
			if _isCrit then
				_data.crit = _data.crit + _value
			else
				_data.damage = _data.damage + _value
			end
			_data.hitCount = _data.hitCount + 1
		end
	end
	
	return _data
end

local function SortDataList(list1, list2)
	if list1.total > list2.total then
		return true
	end			
end

local function GetTargetData(targetList, _targetUnitId)
	for id, targetData in pairs(targetList) do
		if targetData.targetUnitId == _targetUnitId then
			return targetData
		end
	end
	
	return nil
end

local function GetAbilityData(_targetData, _abilityId)
	if _targetData then
		for id, abilityData in pairs(_targetData.abilities) do
			if abilityData.abilityId == _abilityId then
				return abilityData
			end			
		end
	end
	
	return nil
end

local function IsAbilitySlotted(_abilityId)
	for slotId = 3, 8 do
		local slotName = GetSlotName(slotId)
		local abilityName = GetAbilityName(_abilityId)
	
		if slotName == abilityName then
			return true	
		end	
	end
	
	return false
end

local function IsUnitOrGroupInCombat()
	local isPlayerGrouped = IsUnitGrouped(AUI_PLAYER_UNIT_TAG)
	local inCombat = false

	if isPlayerGrouped then
		for i = 1, GetGroupSize(), 1 do
			local unitTag = GetGroupUnitTagByIndex(i)
			inCombat = IsUnitInCombat(unitTag)
			
			if inCombat then
				return true
			end
		end
	else
		inCombat = IsUnitInCombat(AUI_PLAYER_UNIT_TAG)	
	end
	
	return inCombat
end

local function OnUpdate()
	if not isCombatMeterStarted then
		return
	end

	if playerData then
		local ms = GetGameTimeMilliseconds() 
		local endTimeMS = ms - playerData.startTimeMS
		local endTimeS = AUI.Time.MS_To_S(endTimeMS)
		local endTimeSInteger = AUI.Math.Round(endTimeS)

		if playerData.endTimeSInteger ~= endTimeSInteger then
			UpdateUI()
		end	

		if AUI.Combat.Minimeter.IsEnabled() then
			AUI.Combat.Minimeter.UpdateCombatTime(endTimeMS)
		end		
			
		playerData.endTimeSInteger = endTimeSInteger
		
		if not IsUnitOrGroupInCombat() and playerData.init then	
			StopCombatMeter()
		end		
	end	
end

function StartCombatMeter()
	ClearCombatData()
	isCombatMeterStarted = true
		
	OnUpdate()	
		
	EVENT_MANAGER:UnregisterForUpdate("AUI_Combat_OnUpdate")
	EVENT_MANAGER:RegisterForUpdate("AUI_Combat_OnUpdate", 10, OnUpdate)		
end

function StopCombatMeter()
	EVENT_MANAGER:UnregisterForUpdate("AUI_Combat_OnUpdate")
	UpdateUI()
	isCombatMeterStarted = false
	
	combatMeterStoppedTime = GetGameTimeMilliseconds()
end

function AUI.Combat.GetPlayerPetOutData()
	local total = 0
	for sourceId, unitData in pairs(AUI.Combat.Data) do
		if unitData.init and unitData.unitType == COMBAT_UNIT_TYPE_PLAYER_PET then
			return unitData
		end
	end
	
	return nil
end

function AUI.Combat.GetPlayerOutData()
	local total = 0
	local petData = AUI.Combat.GetPlayerPetOutData()
	local playerData = nil
	
	for sourceId, unitData in pairs(AUI.Combat.Data) do
		if unitData.init and unitData.unitType == COMBAT_UNIT_TYPE_PLAYER then
			playerData = AUI.Table.Copy(unitData)

			if playerData then
				for combatType, _ in pairs(playerData) do				
					if combatType == AUI_COMBAT_DATA_TYPE_DAMAGE_OUT or combatType == AUI_COMBAT_DATA_TYPE_HEAL_OUT then
						if playerData[combatType] and petData and petData[combatType] then
							local total = playerData[combatType].total + petData[combatType].total
							local damage = playerData[combatType].damage + petData[combatType].damage
							local crit = playerData[combatType].crit + petData[combatType].crit
							local hitCount = playerData[combatType].hitCount + petData[combatType].hitCount
										
							local startTimeMS = playerData[combatType].startTimeMS
							
							if playerData[combatType].startTimeMS > petData[combatType].startTimeMS then
								startTimeMS = petData[combatType].startTimeMS
							end
							
							local endTimeMS = playerData.endTimeMS
							
							if playerData[combatType].endTimeMS < petData[combatType].endTimeMS then
								endTimeMS = petData[combatType].endTimeMS
							end							
							
							playerData[combatType].total = total
							playerData[combatType].damage = damage
							playerData[combatType].crit = crit
							playerData[combatType].hitCount = hitCount
							playerData[combatType].startTimeMS = startTimeMS
							playerData[combatType].endTimeMS = endTimeMS
							
							break
						end
					end
				end
			end
		end
	end

	return playerData
end

function AUI.Combat.GetPlayerInData()	
	for sourceId, unitData in pairs(AUI.Combat.Data) do
		if unitData.init and unitData.unitType == COMBAT_UNIT_TYPE_PLAYER then
			return AUI.Combat.GetInData(sourceId)
		end
	end
	
	return nil
end

function AUI.Combat.GetOutData(_sourceId)
	if AUI.Combat.Data[_sourceId] and AUI.Combat.Data[_sourceId].init then
		return AUI.Combat.Data[_sourceId]
	end
	
	return nil
end

function AUI.Combat.GetInData(_sourceId)
	local targetList = {}
	
	local isValid = false
	local data = AUI.Table.Copy(AUI.Combat.Data)
	if data then
		for sourceUnitId, unitData in pairs(AUI.Table.Copy(AUI.Combat.Data)) do	
			if unitData.init and sourceUnitId == _sourceId then		
				for combatType, combatData in pairs(unitData) do
					if combatType == AUI_COMBAT_DATA_TYPE_DAMAGE_IN or combatType == AUI_COMBAT_DATA_TYPE_HEAL_IN then			
						targetList[combatType] = {}
						targetList[combatType].targets = {}
						targetList[combatType].total = 0
						targetList[combatType].damage = 0
						targetList[combatType].crit = 0
						targetList[combatType].hitCount= 0
						targetList[combatType].startTimeMS = 0
						targetList[combatType].endTimeMS = 0	
						
						for targetUnitId, targetData in pairs(combatData.targets) do
							targetList[combatType].total = targetList[combatType].total + targetData.total
							targetList[combatType].damage = targetList[combatType].damage + targetData.damage
							targetList[combatType].crit = targetList[combatType].crit + targetData.crit
							targetList[combatType].hitCount = targetList[combatType].hitCount + targetData.hitCount
							
							if targetList[combatType].startTimeMS > targetData.startTimeMS then			
								targetList[combatType].startTimeMS = targetData.startTimeMS
							end
							
							if targetList[combatType].endTimeMS < targetData.endTimeMS then			
								targetList[combatType].endTimeMS = targetData.endTimeMS
							end					
						
							targetList[combatType].targets[targetUnitId] = targetData
							isValid = true
							
							break
						end
					end
				end
			end	
		end
	end
	
	if isValid then
		return targetList
	else
		return nil
	end
end

local function GetCombatString(_unitData, _type)
	local str = ""

	if _unitData and _unitData[_type] and _unitData.startTimeMS > 0 then
	
		local endTimeS =  AUI.Time.MS_To_S(_unitData.endTimeMS)
		local time = 0	

		if endTimeS > 1 then
			time = AUI.Time.GetFormatedString(endTimeS, 1)
		elseif endTimeS > 0.1 then
			time = AUI.Time.GetFormatedString(endTimeS, 2)	
		else
			time = AUI.Time.GetFormatedString(endTimeS, 3)
		end
		
		local total = _unitData[_type].total
		local avarage = AUI.Combat.CalculateDPS(total, _unitData[_type].endTimeMS)			

		if _type == AUI_COMBAT_DATA_TYPE_DAMAGE_OUT then
			if total > 0 then
				str = str .. " - Damage (Out): " .. AUI.String.ToFormatedNumber(total) .. " (" .. time .. ")"
			end
		
			if avarage > 0 then
				str = str .. " - DPS (Out): " .. AUI.String.ToFormatedNumber(avarage)
			end		
		end

		if _type == AUI_COMBAT_DATA_TYPE_DAMAGE_IN then
			if total > 0 then
				str = str .. " - Damage (In): " .. AUI.String.ToFormatedNumber(total) .. " (" .. time .. ")"
			end
		
			if avarage > 0 then
				str = str .. " - DPS (In): " .. AUI.String.ToFormatedNumber(avarage)
			end		
		end			
		
		if _type == AUI_COMBAT_DATA_TYPE_HEAL_OUT then
			if total > 0 then
				str = str .. " - Heal (Out): " .. AUI.String.ToFormatedNumber(total) .. " (" .. time .. ")"
			end

			if avarage > 0 then
				str = str .. " - HPS (Out): " .. AUI.String.ToFormatedNumber(avarage)
			end				
		end				

		if _type == AUI_COMBAT_DATA_TYPE_HEAL_IN then
			if total > 0 then
				str = str .. " - Heal (In): " .. AUI.String.ToFormatedNumber(total) .. " (" .. time .. ")"
			end

			if avarage > 0 then
				str = str .. " - HPS (In): " .. AUI.String.ToFormatedNumber(avarage)
			end				
		end				
	end
	
	return str
end

function AUI.Combat.GetHighestTarget(_playerUnitData)
	if not _playerUnitData then
		return nil
	end

	local lastTargetData = nil

	for targetUnitId, targetData in pairs(_playerUnitData.targets) do
		if not lastTargetData or targetData.total > lastTargetData.total then
			lastTargetData = targetData
		end
	end
		
	return lastTargetData
end

function AUI.Combat.PostHighestTargetCombatStatistics()
	local totalDamage = 0
	local totalheal = 0

	local unitData = AUI.Combat.GetPlayerOutData()	
	if unitData then
		if unitData[AUI_COMBAT_DATA_TYPE_DAMAGE_OUT] or unitData[AUI_COMBAT_DATA_TYPE_HEAL_OUT] then
			if unitData[AUI_COMBAT_DATA_TYPE_DAMAGE_OUT] then
				totalDamage = unitData[AUI_COMBAT_DATA_TYPE_DAMAGE_OUT].total
			end
			
			if unitData[AUI_COMBAT_DATA_TYPE_HEAL_OUT] then
				totalheal = unitData[AUI_COMBAT_DATA_TYPE_HEAL_OUT].total
			end

			if totalDamage > totalheal then
				local targetData = AUI.Combat.GetHighestTarget(unitData[AUI_COMBAT_DATA_TYPE_DAMAGE_OUT])
				AUI.Combat.PostCombatStatistics(unitData, AUI_COMBAT_DATA_TYPE_DAMAGE_OUT, targetData)
			else
				local targetData = AUI.Combat.GetHighestTarget(unitData[AUI_COMBAT_DATA_TYPE_HEAL_OUT])	
				AUI.Combat.PostCombatStatistics(unitData, AUI_COMBAT_DATA_TYPE_HEAL_OUT, targetData)
			end
		else
			d("AUI - No combat statistic available")	
		end
	else
		d("AUI - No combat statistic available")	
	end
end

function AUI.Combat.PostPlayerCombatStatistics()		
	AUI.Combat.PostCombatStatistics(playerData)	
end

function AUI.Combat.PostCombatStatistics(_unitData, _type, _targetData)	
	local str = "AUI "

	if not _unitData then
		d(str .. " - No combat statistic available")	
		return
	end	
	
	if not _type and not _targetData then
		str = str .. GetCombatString(_unitData, AUI_COMBAT_DATA_TYPE_DAMAGE_OUT)
		str = str .. GetCombatString(_unitData, AUI_COMBAT_DATA_TYPE_HEAL_OUT)
		str = str .. GetCombatString(_unitData, AUI_COMBAT_DATA_TYPE_DAMAGE_IN)
		str = str .. GetCombatString(_unitData, AUI_COMBAT_DATA_TYPE_HEAL_IN)
	elseif _type then
		str = str .. " Source: " .. zo_strformat(SI_UNIT_NAME, _unitData.unitName)
		
		if _targetData then
			str = str .. " => Target: " .. AUI.Combat.Data[_targetData.targetUnitId].unitName
			str = str .. GetCombatString(_targetData, _type)
		else
			str = str .. GetCombatString(_unitData, _type)
		end	
	end
	
	StartChatInput(str, CHAT_CHANNEL_PARTY , nil)
end

function AUI.Combat.CalculateDPS(_totalDamage, _time)
	local average = 0

	if _totalDamage and _time then
		local value = _totalDamage
		local s = _time / 1000

		if s > 0 then
			average = AUI.Math.Round(_totalDamage / s)
		end
	end
	
	if average >= _totalDamage then
		average = _totalDamage
	end
	
	return average
end

function AUI.Combat.CalculateCritPrecent(_totalDamage, _totalCrit)
	return (_totalCrit / _totalDamage) * 100
end

function AUI.Combat.GetTotalValue(_targetList, _index)
	local value = 0

	if _targetList and _index then
		for targetId, targetData in pairs(_targetList) do
			value = value + targetData[_index]
		end	
	end
	
	return value
end

function AUI.Combat.GetTotalAbilityList(_targetList)
	local abilityList = {}

	for _, targetData in pairs(_targetList) do					
		for _, abilityData in pairs(targetData.abilities) do
			local newAbilityData = abilityList[abilityData.abilityName]
			if not newAbilityData then
				abilityList[abilityData.abilityName] = {total = 0, crit = 0, hitCount = 0, abilityName = abilityData.abilityName, abilityId = abilityData.abilityId, icon = abilityData.icon}
				newAbilityData = abilityList[abilityData.abilityName]
			end
					
			newAbilityData.total = newAbilityData.total + abilityData.total
			newAbilityData.crit = newAbilityData.crit + abilityData.crit
			newAbilityData.hitCount = newAbilityData.hitCount + abilityData.hitCount
		end	
	end		
	
	return abilityList
end

local function CreateData(_unitId, _unitType, _combatType)
	if not _unitId or not _unitType or not _combatType then
		return nil
	end	

	if not AUI.Combat.Data[_unitId] then	
		AUI.Combat.Data[_unitId] = {}	
		AUI.Combat.Data[_unitId].init = false
		AUI.Combat.Data[_unitId].valid = false
		AUI.Combat.Data[_unitId].unitName = "Unit: " .. _unitId
		AUI.Combat.Data[_unitId].sourceUnitId = _unitId
		AUI.Combat.Data[_unitId].unitType = _unitType		
		AUI.Combat.Data[_unitId].startTimeMS = 0
		AUI.Combat.Data[_unitId].endTimeMS = 0			
	end
	
	if not AUI.Combat.Data[_unitId][_combatType] then
		AUI.Combat.Data[_unitId][_combatType] = {}	
		AUI.Combat.Data[_unitId][_combatType].targets = {}	
		AUI.Combat.Data[_unitId][_combatType].unitName = AUI.Combat.Data[_unitId].unitName
		AUI.Combat.Data[_unitId][_combatType].total = 0
		AUI.Combat.Data[_unitId][_combatType].damage = 0
		AUI.Combat.Data[_unitId][_combatType].crit = 0
		AUI.Combat.Data[_unitId][_combatType].hitCount = 0
		AUI.Combat.Data[_unitId][_combatType].startTimeMS = 0
		AUI.Combat.Data[_unitId][_combatType].endTimeMS = 0		
	end	
			
	if idList[_unitId] then 
		AUI.Combat.Data[_unitId].unitName = idList[_unitId].unitName
		AUI.Combat.Data[_unitId][_combatType].unitName = AUI.Combat.Data[_unitId].unitName
		
		if IsPlayer(idList[_unitId].unitType) or IsGroup(idList[_unitId].unitType) then
			AUI.Combat.Data[_unitId].valid = true	
		end
	end	
	
	return AUI.Combat.Data[_unitId]
end	

	
local function AddData(_sourceUnitId, _sourceType, _targetUnitId, _targetType, _hitValue, _isCrit, _abilityName, _abilityId, _abilityTexture, _combatType)
	if not _sourceUnitId or not _sourceType or not _targetUnitId or not _combatType then
		return false
	end
	
	local data = CreateData(_sourceUnitId, _sourceType, _combatType)
	if data then
		local ms = GetGameTimeMilliseconds()
	
		if data.startTimeMS == 0 then
			data.startTimeMS = ms
		end	
	
		if data[_combatType].startTimeMS == 0 then
			data[_combatType].startTimeMS = ms
		end	

		UpdateData(_hitValue, _isCrit, data[_combatType], ms)

		if not data[_combatType].targets[_targetUnitId] then
			data[_combatType].targets[_targetUnitId] = {sourceUnitId = _sourceUnitId, total = 0, damage = 0, crit= 0, startTimeMS = ms, endTimeMS = 0, hitCount = 0, targetUnitId = _targetUnitId, abilities = {}}
		end		

		UpdateData(_hitValue, _isCrit, data[_combatType].targets[_targetUnitId], ms)

		if data[_combatType].targets[_targetUnitId] then
			if not data[_combatType].targets[_targetUnitId].abilities[_abilityId] then
				if AUI.String.IsEmpty(_abilityName) then
					_abilityName = AUI.L10n.GetString("unknown")
				end		
			
				data[_combatType].targets[_targetUnitId].abilities[_abilityId] = {total = 0, damage = 0, crit= 0, startTimeMS = ms, endTimeMS = 0, hitCount = 0, abilityName = _abilityName, abilityId = _abilityId, icon = _abilityTexture}
			end

			UpdateData(_hitValue, _isCrit, data[_combatType].targets[_targetUnitId].abilities[_abilityId] , ms)				
		end

		if data.valid then
			data.init = true
			
			if _sourceType == COMBAT_UNIT_TYPE_PLAYER then
				playerData = data		
			end
			
			return true
		end	
	end
end	

function AUI.Combat.OnCombatEvent(_eventCode, _result, _isError, _abilityName, _abilityGraphic, _abilityActionSlotType, _sourceName, _sourceType, _targetName, _targetType, _hitValue, _powerType, _damageType, _log, _sourceUnitId, _targetUnitId, _abilityId)
	if not isLoaded or _hitValue <= 0 or _isError or _sourceUnitId <= 0 or _targetUnitId <= 0 then
		return
	end			
	
	if not idList[_sourceUnitId] then
		if not AUI.String.IsEmpty(_sourceName) then
			idList[_sourceUnitId] = {
				["unitName"] = AUI.String.FormatName(_sourceName),
				["unitType"] = _sourceType,
			}
		end
	end
	
	if not idList[_targetUnitId] then
		if not AUI.String.IsEmpty(_targetName) then
			idList[_targetUnitId] = {
				["unitName"] = AUI.String.FormatName(_targetName),
				["unitType"] = _targetType,
			}
		end
	end	

	if not isCombatMeterStarted then
		if combatMeterStoppedTime + 40 > GetGameTimeMilliseconds() then
			isCombatMeterStarted = true
		
			EVENT_MANAGER:UnregisterForUpdate("AUI_Combat_OnUpdate")
			EVENT_MANAGER:RegisterForUpdate("AUI_Combat_OnUpdate", 40, OnUpdate)
		elseif (IsUnitOrGroupInCombat() or isDamage[_result] and IsPlayer(_sourceType)) then
			StartCombatMeter()	
		end			
	end
	
	if isDamage[_result] then	
		local abilityTexture = GetAbilityIcon(_abilityId) or "/esoui/art/icons/icon_missing.dds"
		local abilityName = AUI.String.FirstToUpper(zo_strformat(SI_UNIT_NAME, _abilityName))
		local isCrit = IsCrit(_result)
			
		if isCombatMeterStarted then
			AddData(_sourceUnitId, _sourceType, _targetUnitId, _targetType, _hitValue, isCrit, abilityName, _abilityId, abilityTexture, AUI_COMBAT_DATA_TYPE_DAMAGE_OUT)
			AddData(_targetUnitId, _targetType, _sourceUnitId, _sourceType, _hitValue, isCrit, abilityName, _abilityId, abilityTexture, AUI_COMBAT_DATA_TYPE_DAMAGE_IN)
		end
		
		if AUI.Combat.Text.IsEnabled() then
			if IsPlayer(_sourceType) and not IsPlayer(_targetType) then				
				if isCrit then	
					if AUI.Settings.Combat.scrolling_text_show_critical_damage_out then
						AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_CRIT_DAMAGE_OUT, AUI_SCROLLING_TEXT_PANEL_OUT, abilityTexture, abilityName)
					end
				elseif AUI.Settings.Combat.scrolling_text_show_damage_out then
					AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_DAMAGE_OUT, AUI_SCROLLING_TEXT_PANEL_OUT, abilityTexture, abilityName)
				end
			elseif IsPlayer(_targetType) and not IsPlayer(_sourceType) then	
				if isCrit then
					if AUI.Settings.Combat.scrolling_text_show_critical_damage_in then
						AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_CRIT_DAMAGE_IN, AUI_SCROLLING_TEXT_PANEL_IN, abilityTexture, abilityName)	
					end
				elseif AUI.Settings.Combat.scrolling_text_show_damage_in then
					AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_DAMAGE_IN, AUI_SCROLLING_TEXT_PANEL_IN, abilityTexture, abilityName)
				end				
			end
		end
	elseif isHeal[_result] then
		local abilityTexture = GetAbilityIcon(_abilityId) or "/esoui/art/icons/icon_missing.dds"
		local abilityName = AUI.String.FirstToUpper(zo_strformat(SI_UNIT_NAME, _abilityName))
		local isCrit = IsCrit(_result)	
	
		if isCombatMeterStarted then
			AddData(_sourceUnitId, _sourceType, _targetUnitId, _targetType, _hitValue, isCrit, abilityName, _abilityId, abilityTexture, AUI_COMBAT_DATA_TYPE_HEAL_OUT)
			AddData(_targetUnitId, _targetType, _sourceUnitId, _sourceType, _hitValue, isCrit, abilityName, _abilityId, abilityTexture, AUI_COMBAT_DATA_TYPE_HEAL_IN)
		end
		
		if AUI.Combat.Text.IsEnabled() then	
			if IsPlayer(_sourceType) then		
				if isCrit then
					if AUI.Settings.Combat.scrolling_text_show_critical_heal_out then
						AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_CRIT_HEAL_OUT, AUI_SCROLLING_TEXT_PANEL_OUT, abilityTexture, abilityName)
					end
				elseif AUI.Settings.Combat.scrolling_text_show_heal_out then
					AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_HEAL_OUT, AUI_SCROLLING_TEXT_PANEL_OUT, abilityTexture, abilityName)
				end	
			elseif IsPlayer(_targetType)  then
				if isCrit then
					if AUI.Settings.Combat.scrolling_text_show_critical_heal_in then
						AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_CRIT_HEAL_IN, AUI_SCROLLING_TEXT_PANEL_IN, abilityTexture, abilityName)
					end
				elseif AUI.Settings.Combat.scrolling_text_show_heal_in then
					AUI.Combat.Text.InsertMessage(_hitValue, AUI_SCROLLING_TEXT_HEAL_IN, AUI_SCROLLING_TEXT_PANEL_IN, abilityTexture, abilityName)
				end	
			end		
		end
	end	
end

function AUI.Combat.OnUltimateUpdate(_unitTag, _powerValue, _powerMax, _powerEffectiveMax)
	if not isLoaded then
		return
	end

	if AUI.Combat.Text.IsEnabled() and AUI.Settings.Combat.scrolling_text_show_ultimate_ready and _unitTag == AUI_PLAYER_UNIT_TAG then
		local ultimateSlot = GetSlotAbilityCost(8)
		if ultimateSlot > 0 then 
			local unitPercent = AUI.Math.Round((_powerValue / ultimateSlot) * 100)
			if unitPercent >= 100 then			
				if ultiReady then
					AUI.Combat.Text.InsertMessage(AUI.L10n.GetString("ultimate_ready"), AUI_SCROLLING_TEXT_ULTI_READY, AUI_SCROLLING_TEXT_PANEL_REPORT)
				end			
				ultiReady = false
			else
				ultiReady = true
			end
		end		
	end
end

function AUI.Combat.OnActionUpdateCooldowns()
	if not isLoaded then
		return
	end

	if AUI.Combat.Text.IsEnabled() then
		if potionReady or potionReady then
			local currentQuickSlot = GetCurrentQuickslot()
			local _, _, canUse = GetSlotCooldownInfo(currentQuickSlot)
		
			if canUse then	
				AUI.Combat.Text.InsertMessage(AUI.L10n.GetString("potion_ready"), AUI_SCROLLING_TEXT_POTION_READY, AUI_SCROLLING_TEXT_PANEL_REPORT)
				potionReady = false
			end
		end
	end
end

function AUI.Combat.OnInventoryItemUsed(_itemSoundCategory)
	if not isLoaded then
		return
	end

	if AUI.Combat.Text.IsEnabled() then
		if _itemSoundCategory == ITEM_SOUND_CATEGORY_POTION then 
			potionReady = true
		end
	end
end

function AUI.Combat.OnEffectChanged(_changeType, _effectSlot, _effectName, _unitTag, _beginTime, _endTime, _stackCount, _iconName, _buffType, _effectType, _abilityType, _statusEffectType, _unitName, _unitId, _abilityId)
	if not isLoaded or _unitTag ~= AUI_PLAYER_UNIT_TAG then
		return
	end

	if AUI.Combat.Text.IsEnabled() and IsAbilitySlotted(_abilityId) then
		if _changeType == 1 then
			if AUI.Settings.Combat.scrolling_text_show_instant_casts and AUI.Ability.IsProc(_abilityId) then
				if not abilityProcEffects[_abilityId] then
					local effectName = zo_strformat(SI_UNIT_NAME, _effectName)
					AUI.Combat.Text.InsertMessage(effectName, AUI_SCROLLING_TEXT_PROC, AUI_SCROLLING_TEXT_PANEL_REPORT)
					abilityProcEffects[_abilityId] = true
				end
			end
		elseif _changeType == 2 then
			if _abilityType == ABILITY_TYPE_BONUS then
				abilityProcEffects[_abilityId] = nil
			end
		end
	end
end

local function UpdateExperienceData()
	experienceData.exp = GetUnitXP(AUI_PLAYER_UNIT_TAG)
	experienceData.cxp = GetPlayerChampionXP()	
end

function AUI.Combat.OnLevelUpdate(_eventCode, _unitTag, _level)
	UpdateExperienceData()
end

function AUI.Combat.OnVeteranRankUpdate(_eventCode, _unitTag, _rank)
	UpdateExperienceData()
end

function AUI.Combat.OnExperienceUpdate(_unitTag, _currentExp, _maxExp, _reason)
	if not isLoaded then
		return
	end
	if _unitTag == AUI_PLAYER_UNIT_TAG  then
		local unitData = AUI.Combat.Data[AUI_PLAYER_UNIT_TAG]
		if experienceData then
			if AUI.Settings.Combat.scrolling_text_show_exp then
				local expierence = math.max(_currentExp - experienceData.exp, 0)
				if expierence > 0 then
					AUI.Combat.Text.InsertMessage(expierence, AUI_SCROLLING_TEXT_EXP, AUI_SCROLLING_TEXT_PANEL_REPORT)
				end
			end
				
			if AUI.Settings.Combat.scrolling_text_show_cxp then
				local currentCxp = GetPlayerChampionXP()
				local cxp = math.max(currentCxp - experienceData.cxp, 0)	
				if cxp > 0 then
					AUI.Combat.Text.InsertMessage(cxp, AUI_SCROLLING_TEXT_CXP, AUI_SCROLLING_TEXT_PANEL_REPORT)	
				end
			end
		end
	end
	
	UpdateExperienceData()
end	

function AUI.Combat.OnAlliancePointUpdate(_alliancePoints, _difference)
	if not isLoaded then
		return
	end

	if AUI.Combat.Text.IsEnabled() and AUI.Settings.Combat.scrolling_text_show_ap then
		if _difference > 0 then
			AUI.Combat.Text.InsertMessage(_difference, AUI_SCROLLING_TEXT_AP, AUI_SCROLLING_TEXT_PANEL_REPORT)
		end
	end
end	

function AUI.Combat.OnTelVarStoneUpdate(_newTelvarStones, _oldTelvarStones)
	if not isLoaded then
		return
	end

	if AUI.Combat.Text.IsEnabled() and AUI.Settings.Combat.scrolling_text_show_telvar then
		local difference = _newTelvarStones - _oldTelvarStones
		if difference > 0 then
			AUI.Combat.Text.InsertMessage(difference, AUI_SCROLLING_TEXT_TELVAR, AUI_SCROLLING_TEXT_PANEL_REPORT)
		end
	end
end	

function AUI.Combat.OnPlayerCombatState(_inCombat)
	if not isLoaded then
		return
	end

	if _inCombat then
		if AUI.Combat.Text.IsEnabled() and AUI.Settings.Combat.scrolling_text_show_combat_start then
			AUI.Combat.Text.InsertMessage(AUI.L10n.GetString("combat_start"), AUI_SCROLLING_TEXT_COMBAT_START, AUI_SCROLLING_TEXT_PANEL_REPORT)
		end
	else
		if AUI.Combat.Text.IsEnabled() and AUI.Settings.Combat.scrolling_text_show_combat_end then
			AUI.Combat.Text.InsertMessage(AUI.L10n.GetString("combat_end"), AUI_SCROLLING_TEXT_COMBAT_END, AUI_SCROLLING_TEXT_PANEL_REPORT)
		end
	end	
end
	
function AUI.Combat.OnGroupUpdate()
	if not isLoaded then
		return
	end

	if AUI.Combat.DamageMeter.IsEnabled() then
		AUI.Combat.DamageMeter.UpdateUI()
	end
end	
	
function AUI.Combat.Load()
	if isLoaded then
		return
	end

	isLoaded = true
	
	AUI.Combat.SetMenuData()

	if AUI.Combat.Minimeter.IsEnabled() then	
		AUI.Combat.Minimeter.Load()
	end

	if AUI.Combat.Text.IsEnabled() then	
		AUI.Combat.Text.Load()
	end	
	
	if AUI.Combat.DamageMeter.IsEnabled() then	
		AUI.Combat.DamageMeter.Load()
	end		
	
	UpdateExperienceData()	
	
	AUI.Combat.Statistics.Load()
end

function AUI.Combat.Minimeter.IsEnabled()
	return AUI.Settings.Combat.minimeter_enabled
end

function AUI.Combat.Text.IsEnabled()
	return AUI.Settings.Combat.scrolling_text_enabled
end

function AUI.Combat.DamageMeter.IsEnabled()
	return AUI.Settings.Combat.damage_meter_enabled
end